@@ -1,43 +1,45 @@
-# Copyright (C) 2008-2010, Sebastian Riedel.
-
 package Mojo::Message;
 
 use strict;
 use warnings;
 
-use base 'Mojo::Stateful';
-use overload '""' => sub { shift->to_string }, fallback => 1;
+use base 'Mojo::Base';
 
 use Carp 'croak';
 use Mojo::Asset::Memory;
-use Mojo::ByteStream 'b';
 use Mojo::Content::Single;
 use Mojo::Loader;
 use Mojo::Parameters;
 use Mojo::Upload;
+use Mojo::Util qw/decode url_unescape/;
 
-use constant CHUNK_SIZE => $ENV{MOJO_CHUNK_SIZE} || 8192;
+use constant CHUNK_SIZE => $ENV{MOJO_CHUNK_SIZE} || 262144;
 
-__PACKAGE__->attr(buffer  => sub { Mojo::ByteStream->new });
 __PACKAGE__->attr(content => sub { Mojo::Content::Single->new });
 __PACKAGE__->attr(default_charset => 'UTF-8');
 __PACKAGE__->attr(dom_class       => 'Mojo::DOM');
-__PACKAGE__->attr([qw/finish_cb progress_cb/]);
-__PACKAGE__->attr(json_class                        => 'Mojo::JSON');
-__PACKAGE__->attr([qw/major_version minor_version/] => 1);
+__PACKAGE__->attr(json_class      => 'Mojo::JSON');
+__PACKAGE__->attr(max_line_size => sub { $ENV{MOJO_MAX_LINE_SIZE} || 10240 });
+__PACKAGE__->attr(
+    max_message_size => sub { $ENV{MOJO_MAX_MESSAGE_SIZE} || 5242880 });
+__PACKAGE__->attr([qw/on_finish on_progress/]);
+__PACKAGE__->attr(version => '1.1');
+
+# DEPRECATED in Comet!
+*finish_cb   = \&on_finish;
+*progress_cb = \&on_progress;
 
 # I'll keep it short and sweet. Family. Religion. Friendship.
 # These are the three demons you must slay if you wish to succeed in
 # business.
 sub at_least_version {
     my ($self, $version) = @_;
-    my ($major, $minor) = split /\./, $version;
+    my ($sma,  $smi)     = split /\./, $version;
+    my ($cma,  $cmi)     = split /\./, $self->version;
 
     # Version is equal or newer
-    return 1 if $major < $self->major_version;
-    if ($major == $self->major_version) {
-        return 1 if $minor <= $self->minor_version;
-    }
+    return 1 if $sma < $cma;
+    return 1 if $sma == $cma && $smi <= $cmi;
 
     # Version is older
     return;
@@ -50,34 +52,37 @@ sub body {
     $self->content(Mojo::Content::Single->new)
       if $self->content->isa('Mojo::Content::MultiPart');
 
+    # Content
+    my $content = $self->content;
+
     # Get
     unless (@_) {
-        return $self->body_cb
-          ? $self->body_cb
+        return $content->on_read
+          ? $content->on_read
           : return $self->content->asset->slurp;
     }
 
     # New content
-    my $content = shift;
+    my $new = shift;
 
     # Cleanup
-    $self->body_cb(undef);
-    $self->content->asset(Mojo::Asset::Memory->new);
+    $content->on_read(undef);
+    $content->asset(Mojo::Asset::Memory->new);
 
     # Shortcut
-    return $self unless defined $content;
+    return $self unless defined $new;
 
     # Callback
-    if (ref $content eq 'CODE') { $self->body_cb($content) }
+    if (ref $new eq 'CODE') {
+        $content->on_read(sub { shift and $self->$new(@_) });
+    }
 
     # Set text content
-    elsif (length $content) { $self->content->asset->add_chunk($content) }
+    elsif (length $new) { $content->asset->add_chunk($new) }
 
     return $self;
 }
 
-sub body_cb { shift->content->body_cb(@_) }
-
 sub body_params {
     my $self = shift;
 
@@ -121,22 +126,6 @@ sub body_params {
 
 sub body_size { shift->content->body_size }
 
-sub build {
-    my $self    = shift;
-    my $message = '';
-
-    # Start line
-    $message .= $self->build_start_line;
-
-    # Headers
-    $message .= $self->build_headers;
-
-    # Body
-    $message .= $self->build_body;
-
-    return $message;
-}
-
 # My new movie is me, standing in front of a brick wall for 90 minutes.
 # It cost 80 million dollars to make.
 # How do you sleep at night?
@@ -148,7 +137,8 @@ sub build_body {
     my $body = $self->content->build_body(@_);
 
     # Finished
-    if (my $cb = $self->finish_cb) { $self->$cb }
+    $self->{_state} = 'done';
+    if (my $cb = $self->on_finish) { $self->$cb }
 
     return $body;
 }
@@ -239,23 +229,42 @@ sub dom {
 
     # Charset
     my $charset = $self->default_charset;
-    ($self->headers->content_type || '') =~ /charset=\"?(\S+)\"?/
+    ($self->headers->content_type || '') =~ /charset=\"?([^\s;]+)\"?/
       and $charset = $1;
 
     # Parse
-    return $class->new(charset => $charset)->parse($self->body);
+    my $dom = $class->new(charset => $charset)->parse($self->body);
+
+    # Find right away
+    return $dom->find(@_) if @_;
+
+    return $dom;
+}
+
+sub error {
+    my $self = shift;
+
+    # Get
+    unless (@_) {
+        return unless my $error = $self->{_error};
+        return wantarray ? @$error : $error->[0];
+    }
+
+    # Set
+    $self->{_error} = [@_];
+    $self->{_state} = 'done';
+
+    return $self;
 }
 
 sub fix_headers {
     my $self = shift;
 
-    # Content-Length header is required in HTTP 1.0 (and above) messages if
-    # there's a body, sadly many clients are expecting broken server behavior
-    # if the Content-Length header is missing, so we are defaulting to
-    # "Content-Length: 0" which has proven to just work in the real world
+    # Content-Length header is required in HTTP 1.0 (and above)
     if ($self->at_least_version('1.0') && !$self->is_chunked) {
-        $self->headers->content_length($self->body_size)
-          unless $self->headers->content_length;
+        my $headers = $self->headers;
+        $headers->content_length($self->body_size)
+          unless $headers->content_length;
     }
 
     return $self;
@@ -265,42 +274,40 @@ sub get_body_chunk {
     my $self = shift;
 
     # Progress
-    if (my $cb = $self->progress_cb) { $self->$cb('body', @_) }
+    if (my $cb = $self->on_progress) { $self->$cb('body', @_) }
 
     # Chunk
-    if (defined(my $chunk = $self->content->get_body_chunk(@_))) {
-        return $chunk;
-    }
+    my $chunk = $self->content->get_body_chunk(@_);
+    return $chunk if !defined $chunk || length $chunk;
 
-    # Finished
-    if (my $cb = $self->finish_cb) { $self->$cb }
+    # Finish
+    $self->{_state} = 'done';
+    if (my $cb = $self->on_finish) { $self->$cb }
 
-    return;
+    return $chunk;
 }
 
 sub get_header_chunk {
     my $self = shift;
 
     # Progress
-    if (my $cb = $self->progress_cb) { $self->$cb('headers', @_) }
+    if (my $cb = $self->on_progress) { $self->$cb('headers', @_) }
 
     # HTTP 0.9 has no headers
     return '' if $self->version eq '0.9';
 
-    # Fix headers
-    $self->fix_headers;
-
-    $self->content->get_header_chunk(@_);
+    return $self->content->get_header_chunk(@_);
 }
 
 sub get_start_line_chunk {
     my ($self, $offset) = @_;
 
     # Progress
-    if (my $cb = $self->progress_cb) { $self->$cb('start_line', @_) }
+    if (my $cb = $self->on_progress) { $self->$cb('start_line', @_) }
 
-    my $copy = $self->_build_start_line;
-    return substr($copy, $offset, CHUNK_SIZE);
+    # Get chunk
+    my $copy = $self->{_buffer} ||= $self->_build_start_line;
+    return substr $copy, $offset, CHUNK_SIZE;
 }
 
 sub has_leftovers { shift->content->has_leftovers }
@@ -329,6 +336,18 @@ sub headers {
 
 sub is_chunked { shift->content->is_chunked }
 
+sub is_done {
+    return 1 if (shift->{_state} || '') eq 'done';
+    return;
+}
+
+sub is_limit_exceeded {
+    my $self = shift;
+    return unless my $code = ($self->error)[1];
+    return unless $code eq '413';
+    return 1;
+}
+
 sub is_multipart { shift->content->is_multipart }
 
 sub json {
@@ -357,27 +376,26 @@ sub param {
     return $self->{body_params}->param(@_);
 }
 
-sub parse {
-    my ($self, $chunk) = @_;
-
-    # Buffer
-    $self->buffer->add_chunk($chunk) if defined $chunk;
-
-    return $self->_parse(0);
-}
-
-sub parse_until_body {
-    my ($self, $chunk) = @_;
-
-    # Buffer
-    $self->buffer->add_chunk($chunk);
-
-    return $self->_parse(1);
-}
+sub parse            { shift->_parse(0, @_) }
+sub parse_until_body { shift->_parse(1, @_) }
 
 sub start_line_size { length shift->build_start_line }
 
-sub to_string { shift->build(@_) }
+sub to_string {
+    my $self    = shift;
+    my $message = '';
+
+    # Start line
+    $message .= $self->build_start_line;
+
+    # Headers
+    $message .= $self->build_headers;
+
+    # Body
+    $message .= $self->build_body;
+
+    return $message;
+}
 
 sub upload {
     my ($self, $name) = @_;
@@ -442,83 +460,85 @@ sub uploads {
     return \@uploads;
 }
 
-sub version {
-    my ($self, $version) = @_;
-
-    # Return normalized version
-    unless ($version) {
-        my $major = $self->major_version;
-        $major = 1 unless defined $major;
-        my $minor = $self->minor_version;
-        $minor = 1 unless defined $minor;
-        return "$major.$minor";
-    }
-
-    # New version
-    my ($major, $minor) = split /\./, $version;
-    $self->major_version($major);
-    $self->minor_version($minor);
-
-    return $self;
-}
+sub write       { shift->content->write(@_) }
+sub write_chunk { shift->content->write_chunk(@_) }
 
 sub _build_start_line {
     croak 'Method "_build_start_line" not implemented by subclass';
 }
 
 sub _parse {
-    my $self = shift;
-    my $until_body = @_ ? shift : 0;
+    my ($self, $until_body, $chunk) = @_;
 
-    # Progress
-    if (my $cb = $self->progress_cb) { $self->$cb }
+    # Buffer
+    $self->{_buffer}   = '' unless defined $self->{_buffer};
+    $self->{_raw_size} = 0  unless exists $self->{_raw_size};
 
-    # Start line and headers
-    my $buffer = $self->buffer;
-    if ($self->is_state(qw/start headers/)) {
+    # Add chunk
+    if (defined $chunk) {
+        $self->{_raw_size} += length $chunk;
+        $self->{_buffer} .= $chunk;
+    }
+
+    # Start line
+    $self->_parse_start_line unless $self->{_state};
+
+    # Got start line and headers
+    if (!$self->{_state} || $self->{_state} eq 'headers') {
 
         # Check line size
         $self->error('Maximum line size exceeded.', 413)
-          if $buffer->size > ($ENV{MOJO_MAX_LINE_SIZE} || 10240);
+          if length $self->{_buffer} > $self->max_line_size;
     }
 
     # Check message size
     $self->error('Maximum message size exceeded.', 413)
-      if $buffer->raw_size > ($ENV{MOJO_MAX_MESSAGE_SIZE} || 5242880);
+      if $self->{_raw_size} > $self->max_message_size;
 
     # Content
-    if ($self->is_state(qw/content done done_with_leftovers/)) {
+    my $state = $self->{_state} || '';
+    if ($state eq 'body' || $state eq 'content' || $state eq 'done') {
         my $content = $self->content;
 
-        # HTTP 0.9 has no headers
-        $content->state('body') if $self->version eq '0.9';
-
-        # Parse
-        $content->filter_buffer($buffer);
+        # Empty buffer
+        my $buffer = $self->{_buffer};
+        $self->{_buffer} = '';
 
         # Until body
-        if ($until_body) { $self->content($content->parse_until_body) }
+        if ($until_body) {
+            $self->content($content->parse_until_body($buffer));
+        }
 
-        # Whole message
-        else { $self->content($content->parse) }
+        # CGI
+        elsif ($self->{_state} eq 'body') {
+            $self->content($content->parse_body($buffer));
+        }
 
-        # HTTP 0.9 has no defined length
-        $content->state('done') if $self->version eq '0.9';
+        # HTTP 0.9
+        elsif ($self->version eq '0.9') {
+            $self->content($content->parse_body_once($buffer));
+        }
+
+        # Parse
+        else { $self->content($content->parse($buffer)) }
     }
 
     # Done
-    $self->done if $self->content->is_done;
+    $self->{_state} = 'done' if $self->content->is_done;
 
-    # Done with leftovers, maybe pipelined
-    $self->state('done_with_leftovers')
-      if $self->content->is_state('done_with_leftovers');
+    # Progress
+    if (my $cb = $self->on_progress) { $self->$cb }
 
     # Finished
-    if ((my $cb = $self->finish_cb) && $self->is_finished) { $self->$cb }
+    if ((my $cb = $self->on_finish) && $self->is_done) { $self->$cb }
 
     return $self;
 }
 
+sub _parse_start_line {
+    croak 'Method "_parse_start_line" not implemented by subclass';
+}
+
 sub _parse_formdata {
     my $self = shift;
 
@@ -557,16 +577,16 @@ sub _parse_formdata {
         my $value      = $part;
 
         # Unescape
-        $name     = b($name)->url_unescape->to_string;
-        $filename = b($filename)->url_unescape->to_string;
+        url_unescape $name     if $name;
+        url_unescape $filename if $filename;
 
         # Decode
         if ($charset) {
             my $backup = $name;
-            $name     = b($name)->decode($charset)->to_string;
-            $name     = $backup unless defined $name;
-            $backup   = $filename;
-            $filename = b($filename)->decode($charset)->to_string;
+            decode $charset, $name if $name;
+            $name = $backup unless defined $name;
+            $backup = $filename;
+            decode $charset, $filename if $filename;
             $filename = $backup unless defined $filename;
         }
 
@@ -579,7 +599,7 @@ sub _parse_formdata {
             # Decode
             if ($charset && !$part->headers->content_transfer_encoding) {
                 my $backup = $value;
-                $value = b($value)->decode($charset)->to_string;
+                decode $charset, $value;
                 $value = $backup unless defined $value;
             }
         }
@@ -608,35 +628,11 @@ in RFC 2616 and RFC 2388.
 
 =head1 ATTRIBUTES
 
-L<Mojo::Message> inherits all attributes from L<Mojo::Stateful> and
-implements the following new ones.
-
-=head2 C<body_cb>
-
-    my $cb = $message->body_cb;
-
-    $counter = 1;
-    $message = $message->body_cb(sub {
-        my $self  = shift;
-        my $chunk = '';
-        $chunk    = "hello world!" if $counter == 1;
-        $chunk    = "hello world2!\n\n" if $counter == 2;
-        $counter++;
-        return $chunk;
-    });
-
-Content generator callback.
-
-=head2 C<buffer>
-
-    my $buffer = $message->buffer;
-    $message   = $message->buffer(Mojo::ByteStream->new);
-
-Input buffer for parsing.
+L<Mojo::Message> implements the following attributes.
 
 =head2 C<content>
 
-    my $content = $message->content;
+    my $message = $message->content;
     $message    = $message->content(Mojo::Content::Single->new);
 
 Content container, defaults to a L<Mojo::Content::Single> object.
@@ -656,15 +652,6 @@ Default charset used for form data parsing.
 Class to be used for DOM manipulation, defaults to L<Mojo::DOM>.
 Note that this attribute is EXPERIMENTAL and might change without warning!
 
-=head2 C<finish_cb>
-
-    my $cb   = $message->finish_cb;
-    $message = $message->finish_cb(sub {
-        my $self = shift;
-    });
-
-Callback called after message building or parsing is finished.
-
 =head2 C<json_class>
 
     my $class = $message->json_class;
@@ -674,24 +661,35 @@ Class to be used for JSON deserialization with C<json>, defaults to
 L<Mojo::JSON>.
 Note that this attribute is EXPERIMENTAL and might change without warning!
 
-=head2 C<major_version>
+=head2 C<max_line_size>
 
-    my $major_version = $message->major_version;
-    $message          = $message->major_version(1);
+    my $size = $message->max_line_size;
+    $message = $message->max_line_size(1024);
 
-Major version, defaults to C<1>.
+Maximum line size in bytes, defaults to C<10240>.
+Note that this attribute is EXPERIMENTAL and might change without warning!
 
-=head2 C<minor_version>
+=head2 C<max_message_size>
 
-    my $minor_version = $message->minor_version;
-    $message          = $message->minor_version(1);
+    my $size = $message->max_message_size;
+    $message = $message->max_message_size(1024);
 
-Minor version, defaults to C<1>.
+Maximum message size in bytes, defaults to C<5242880>.
+Note that this attribute is EXPERIMENTAL and might change without warning!
 
-=head2 C<progress_cb>
+=head2 C<on_finish>
 
-    my $cb   = $message->progress_cb;
-    $message = $message->progress_cb(sub {
+    my $cb   = $message->on_finish;
+    $message = $message->on_finish(sub {
+        my $self = shift;
+    });
+
+Callback called after message building or parsing is finished.
+
+=head2 C<on_progress>
+
+    my $cb   = $message->on_progress;
+    $message = $message->on_progress(sub {
         my $self = shift;
         print '+';
     });
@@ -700,8 +698,8 @@ Progress callback.
 
 =head1 METHODS
 
-L<Mojo::Message> inherits all methods from L<Mojo::Stateful> and implements
-the following new ones.
+L<Mojo::Message> inherits all methods from L<Mojo::Base> and implements the
+following new ones.
 
 =head2 C<at_least_version>
 
@@ -713,16 +711,7 @@ Check if message is at least a specific version.
 
     my $string = $message->body;
     $message   = $message->body('Hello!');
-
-    $counter = 1;
-    $message = $message->body(sub {
-        my $self  = shift;
-        my $chunk = '';
-        $chunk    = "hello world!" if $counter == 1;
-        $chunk    = "hello world2!\n\n" if $counter == 2;
-        $counter++;
-        return $chunk;
-    });
+    $message   = $message->body(sub {...});
 
 Helper for simplified content access.
 
@@ -738,14 +727,6 @@ C<POST> parameters.
 
 Size of the body in bytes.
 
-=head2 C<to_string>
-
-=head2 C<build>
-
-    my $string = $message->build;
-
-Render whole message.
-
 =head2 C<build_body>
 
     my $string = $message->build_body;
@@ -773,11 +754,22 @@ Access message cookies.
 
 =head2 C<dom>
 
-    my $dom = $message->dom;
+    my $dom        = $message->dom;
+    my $collection = $message->dom('a[href]');
 
-Parses content into a L<Mojo::DOM> object.
+Turns content into a L<Mojo::DOM> object and takes an optional selector to
+perform a C<find> on it right away, which returns a collection.
 Note that this method is EXPERIMENTAL and might change without warning!
 
+=head2 C<error>
+
+    my $message          = $message->error;
+    my ($message, $code) = $message->error;
+    $message             = $message->error('Parser error.');
+    $message             = $message->error('Parser error.', 500);
+
+Parser errors and codes.
+
 =head2 C<fix_headers>
 
     $message = $message->fix_headers;
@@ -806,7 +798,7 @@ Get a chunk of start line data starting from a specific position.
 
     my $leftovers = $message->has_leftovers;
 
-CHeck if message parser has leftover data in the buffer.
+CHeck if message parser has leftover data.
 
 =head2 C<header_size>
 
@@ -827,6 +819,19 @@ Header container, defaults to a L<Mojo::Headers> object.
 
 Check if message content is chunked.
 
+=head2 C<is_done>
+
+    my $done = $message->is_done;
+
+Check if parser is done.
+
+=head2 C<is_limit_exceeded>
+
+    my $limit = $message->is_limit_exceeded;
+
+Check if message has exceeded C<max_line_size> or C<max_message_size>.
+Note that this method is EXPERIMENTAL and might change without warning!
+
 =head2 C<is_multipart>
 
     my $multipart = $message->is_multipart;
@@ -846,7 +851,7 @@ Note that this method is EXPERIMENTAL and might change without warning!
 
     my $bytes = $message->leftovers;
 
-Remove leftover data from the parser buffer.
+Remove leftover data.
 
 =head2 C<param>
 
@@ -873,6 +878,12 @@ Parse message chunk until the body is reached.
 
 Size of the start line in bytes.
 
+=head2 C<to_string>
+
+    my $string = $message->to_string;
+
+Render whole message.
+
 =head2 C<upload>
 
     my $upload  = $message->upload('foo');
@@ -893,6 +904,24 @@ All file uploads.
 
 HTTP version of message.
 
+=head2 C<write>
+
+    $message->write('Hello!');
+    $message->write('Hello!', sub {...});
+
+Write dynamic content, the optional drain callback will be invoked once all
+data has been written.
+Note that this method is EXPERIMENTAL and might change without warning!
+
+=head2 C<write_chunk>
+
+    $message->write_chunk('Hello!');
+    $message->write_chunk('Hello!', sub {...});
+
+Write chunked content, the optional drain callback will be invoked once all
+data has been written.
+Note that this method is EXPERIMENTAL and might change without warning!
+
 =head1 SEE ALSO
 
 L<Mojolicious>, L<Mojolicious::Guides>, L<http://mojolicious.org>.
