@@ -1,5 +1,3 @@
-# Copyright (C) 2008-2010, Sebastian Riedel.
-
 package Mojolicious;
 
 use strict;
@@ -7,19 +5,20 @@ use warnings;
 
 use base 'Mojo';
 
+use Carp 'croak';
 use Mojolicious::Commands;
 use Mojolicious::Plugins;
-use MojoX::Dispatcher::Routes;
-use MojoX::Dispatcher::Static;
-use MojoX::Renderer;
-use MojoX::Session::Cookie;
-use MojoX::Types;
+use Mojolicious::Renderer;
+use Mojolicious::Routes;
+use Mojolicious::Session;
+use Mojolicious::Static;
+use Mojolicious::Types;
 
 __PACKAGE__->attr(controller_class => 'Mojolicious::Controller');
 __PACKAGE__->attr(mode => sub { ($ENV{MOJO_MODE} || 'development') });
 __PACKAGE__->attr(plugins  => sub { Mojolicious::Plugins->new });
-__PACKAGE__->attr(renderer => sub { MojoX::Renderer->new });
-__PACKAGE__->attr(routes   => sub { MojoX::Dispatcher::Routes->new });
+__PACKAGE__->attr(renderer => sub { Mojolicious::Renderer->new });
+__PACKAGE__->attr(routes   => sub { Mojolicious::Routes->new });
 __PACKAGE__->attr(
     secret => sub {
         my $self = shift;
@@ -31,18 +30,48 @@ __PACKAGE__->attr(
         return ref $self;
     }
 );
-__PACKAGE__->attr(session => sub { MojoX::Session::Cookie->new });
-__PACKAGE__->attr(static  => sub { MojoX::Dispatcher::Static->new });
-__PACKAGE__->attr(types   => sub { MojoX::Types->new });
+__PACKAGE__->attr(session => sub { Mojolicious::Session->new });
+__PACKAGE__->attr(static  => sub { Mojolicious::Static->new });
+__PACKAGE__->attr(types   => sub { Mojolicious::Types->new });
 
-our $CODENAME = 'Snowman';
-our $VERSION  = '0.999926';
+our $CODENAME = 'Hot Beverage';
+our $VERSION  = '0.999950';
 
+# These old doomsday devices are dangerously unstable.
+# I'll rest easier not knowing where they are.
+sub AUTOLOAD {
+    my $self = shift;
+
+    # Method
+    my ($package, $method) = our $AUTOLOAD =~ /^([\w\:]+)\:\:(\w+)$/;
+
+    # Helper
+    croak qq/Can't locate object method "$method" via "$package"/
+      unless my $helper = $self->renderer->helper->{$method};
+
+    # Load controller class
+    my $class = $self->controller_class;
+    if (my $e = Mojo::Loader->load($class)) {
+        $self->log->error(
+            ref $e
+            ? qq/Can't load controller class "$class": $e/
+            : qq/Controller class "$class" doesn't exist./
+        );
+    }
+
+    # Run
+    return $class->new(app => $self)->$helper(@_);
+}
+
+sub DESTROY { }
+
+# I personalized each of your meals.
+# For example, Amy: you're cute, so I baked you a pony.
 sub new {
     my $self = shift->SUPER::new(@_);
 
     # Transaction builder
-    $self->build_tx_cb(
+    $self->on_build_tx(
         sub {
             my $self = shift;
 
@@ -50,7 +79,7 @@ sub new {
             my $tx = Mojo::Transaction::HTTP->new;
 
             # Hook
-            $self->plugins->run_hook_reverse(after_build_tx => $tx);
+            $self->plugins->run_hook(after_build_tx => ($tx, $self));
 
             return $tx;
         }
@@ -68,10 +97,6 @@ sub new {
     # Static
     my $static = $self->static;
 
-    # Types
-    $renderer->types($self->types);
-    $static->types($self->types);
-
     # Home
     my $home = $self->home;
 
@@ -80,11 +105,12 @@ sub new {
     $static->root($home->rel_dir('public'));
 
     # Hide own controller methods
-    $r->hide(qw/client cookie finish finished flash helper param pause/);
-    $r->hide(qw/receive_message redirect_to render render_data/);
-    $r->hide(qw/render_exception render_inner render_json render_not_found/);
-    $r->hide(qw/render_partial render_static render_text resume/);
-    $r->hide(qw/send_message session signed_cookie url_for/);
+    $r->hide(qw/AUTOLOAD DESTROY client cookie finish finished flash/);
+    $r->hide(qw/handler helper on_message param redirect_to render/);
+    $r->hide(qw/render_data render_exception render_inner render_json/);
+    $r->hide(qw/render_not_found render_partial render_static render_text/);
+    $r->hide(qw/rendered send_message session signed_cookie url_for/);
+    $r->hide(qw/write write_chunk/);
 
     # Mode
     my $mode = $self->mode;
@@ -115,6 +141,29 @@ sub new {
     return $self;
 }
 
+# Amy, technology isn't intrinsically good or evil. It's how it's used.
+# Like the Death Ray.
+sub defaults {
+    my $self = shift;
+
+    # Initialize
+    $self->{defaults} ||= {};
+
+    # Hash
+    return $self->{defaults} unless @_;
+
+    # Get
+    return $self->{defaults}->{$_[0]} unless @_ > 1 || ref $_[0];
+
+    # Set
+    my $values = ref $_[0] ? $_[0] : {@_};
+    for my $key (keys %$values) {
+        $self->{defaults}->{$key} = $values->{$key};
+    }
+
+    return $self;
+}
+
 # The default dispatchers with exception handling
 sub dispatch {
     my ($self, $c) = @_;
@@ -129,9 +178,11 @@ sub dispatch {
     $self->plugins->run_hook(before_dispatch => $c);
 
     # New request
-    my $path = $c->req->url->path           || '/';
-    my $ua   = $c->req->headers->user_agent || 'Anonymojo';
-    $self->log->debug(qq/*** Request for "$path" from "$ua". ***/);
+    my $req    = $c->req;
+    my $method = $req->method;
+    my $path   = $req->url->path || '/';
+    my $ua     = $req->headers->user_agent || 'Anonymojo';
+    $self->log->debug(qq/$method $path ($ua)./);
 
     # Try to find a static file
     $self->static->dispatch($c);
@@ -145,31 +196,6 @@ sub dispatch {
         # Nothing found
         $c->render_not_found unless $c->res->code;
     }
-
-    # Websocket handshake
-    $c->res->code(101) if !$c->res->code && $c->tx->is_websocket;
-
-    # Finish
-    $self->finish($c);
-}
-
-sub finish {
-    my ($self, $c) = @_;
-
-    # Already finished
-    return if $c->stash->{finished};
-
-    # Paused
-    return if $c->tx->is_paused;
-
-    # Hook
-    $self->plugins->run_hook_reverse(after_dispatch => $c);
-
-    # Session
-    $self->session->store($c);
-
-    # Finished
-    $c->stash->{finished} = 1;
 }
 
 # Bite my shiny metal ass!
@@ -193,16 +219,48 @@ sub handler {
         $tx    = $tx->tx;
     }
 
+    # Defaults
+    my $defaults = $self->defaults;
+    $stash = {%$stash, %$defaults};
+
     # Build default controller and process
     eval {
         $self->process($class->new(app => $self, stash => $stash, tx => $tx));
     };
-    $self->log->error("Processing request failed: $@") if $@;
+
+    # Fatal exception
+    if ($@) {
+        $self->log->fatal("Processing request failed: $@");
+        $tx->res->code(500);
+        $tx->resume;
+    }
+}
+
+sub helper {
+    my $self = shift;
+    my $name = shift;
+
+    # Renderer
+    my $r = $self->renderer;
+
+    # Replace helper
+    $self->log->debug(qq/Helper "$name" already exists, replacing./)
+      if exists $r->helper->{$name};
+
+    # Add helper
+    $r->add_helper($name, @_);
+}
+
+sub hook {
+    my ($self, $name, $cb) = @_;
+
+    # DEPRECATED in Hot Beverage! (callback wrapper)
+    $self->plugins->add_hook($name, sub { shift; $cb->(@_) });
 }
 
 sub plugin {
     my $self = shift;
-    $self->plugins->load_plugin($self, @_);
+    $self->plugins->register_plugin(shift, $self, @_);
 }
 
 # This will run for each request
@@ -257,7 +315,7 @@ Mojolicious - The Web In A Box!
     # Say hello
     sub welcome {
         my $self = shift;
-        $self->render_text('Hi there!');
+        $self->render(text => 'Hi there!');
     }
 
     # Say goodbye from a template (foo/bye.html.ep)
@@ -278,25 +336,45 @@ art technology.
 
 =over 4
 
+=item *
+
 An amazing MVC web framework supporting a simplified single file mode through
 L<Mojolicious::Lite>.
 
+=over 4
+
+Powerful out of the box with RESTful routes, plugins, Perl-ish templates,
+session management, signed cookies, testing framework, static file server,
+I18N, first class unicode support and much more for you to discover.
+
+=back
+
+=item *
+
 Very clean, portable and Object Oriented pure Perl API without any hidden
 magic and no requirements besides Perl 5.8.7.
 
-Full stack HTTP 1.1 and WebSocket client/server implementation with IPv6,
-TLS, IDNA, pipelining, chunking and multipart support.
+=item *
 
-Builtin async IO and prefork web server supporting epoll, kqueue, hot
-deployment and UNIX domain socket sharing, perfect for embedding.
+Full stack HTTP 1.1 and WebSocket client/server implementation with TLS,
+Bonjour, IDNA, Comet (long polling), chunking and multipart support.
 
-CGI, FastCGI and L<PSGI> support.
+=item *
 
-Fresh code, based upon years of experience developing Catalyst.
+Builtin async IO web server supporting epoll, kqueue, UNIX domain sockets and
+hot deployment, perfect for embedding.
 
-Powerful out of the box with RESTful routes, plugins, sessions, signed
-cookies, static file server, testing framework, Perl-ish templates, JSON,
-I18N, first class Unicode support and much more for you to discover!
+=item *
+
+Automatic CGI, FastCGI and L<PSGI> detection.
+
+=item *
+
+JSON and XML/HTML5 parser with CSS3 selector support.
+
+=item *
+
+Fresh code based upon years of experience developing L<Catalyst>.
 
 =back
 
@@ -312,7 +390,7 @@ Web development for humans, making hard things possible and everything fun.
 
     websocket '/echo' => sub {
         my $self = shift;
-        $self->receive_message(
+        $self->on_message(
             sub {
                 my ($self, $message) = @_;
                 $self->send_message("echo: $message");
@@ -324,7 +402,7 @@ Web development for humans, making hard things possible and everything fun.
         my $self = shift;
         my $url  = $self->param('url');
         $self->render(text =>
-              $self->client->get($url)->success->dom->at('title')->text);
+              $self->client->get($url)->res->dom->at('title')->text);
     };
 
     post '/:offset' => sub {
@@ -338,16 +416,14 @@ Web development for humans, making hard things possible and everything fun.
 
     @@ clock.html.ep
     % my ($second, $minute, $hour) = (localtime(time))[0, 1, 2];
-    The time is <%= $hour %>:<%= $minute %>:<%= $second %>.
-
-For more user friendly documentation see L<Mojolicious::Guides> and
-L<Mojolicious::Lite>.
+    <%= link_to clock => begin %>
+        The time is <%= $hour %>:<%= $minute %>:<%= $second %>.
+    <% end %>
 
 =head2 Have Some Cake
 
-    .---------------------------------------------------------------.
-    |                             Fun!                              |
-    '---------------------------------------------------------------'
+Loosely coupled building blocks, use what you like and just ignore the rest.
+
     .---------------------------------------------------------------.
     |                                                               |
     |                .----------------------------------------------'
@@ -364,6 +440,68 @@ L<Mojolicious::Lite>.
     |  CGI  | |  FastCGI  | |  PSGI  | |  HTTP 1.1  | |  WebSocket  |
     '-------' '-----------' '--------' '------------' '-------------'
 
+=head2 Highlights
+
+These are some of the most important building blocks of L<Mojolicious>.
+
+=over 4
+
+=item L<Mojolicious::Lite>
+
+Micro Web Framework built on top of L<Mojolicious> for prototypes and small
+applications.
+
+=item L<Mojo::Client>
+
+Full featured async io HTTP 1.1 and WebSocket client.
+
+=item L<Mojo::DOM>
+
+Very fun and minimalistic XML/HTML5 DOM parser with CSS3 selector support.
+
+=item L<Mojo::JSON>
+
+Minimalistic JSON implementation that just works.
+
+=item L<Mojo::Server::Daemon>
+
+Highly portable async io HTTP 1.1 and WebSocket server, perfect for
+development and testing.
+
+=item L<Mojo::Server::Hypnotoad>
+
+Full featured UNIX optimized preforking async io HTTP 1.1 and WebSocket
+server with support for zero downtime software upgrades (hot deployment).
+
+=item L<Mojo::Server::CGI>, L<Mojo::Server::FastCGI>, L<Mojo::Server::PSGI>
+
+Transparent CGI, FastCGI and PSGI support out of the box.
+
+=item L<Mojo::Template>
+
+Very perlish and minimalistic template system.
+
+=item L<Mojo::ByteStream>
+
+Countless portable and very convenient bytestream manipulation methods.
+
+=item L<Mojolicious::Commands>
+
+Pluggable command line system and the backbone of the C<mojo> script.
+
+=item L<Test::Mojo>
+
+Test driven development toolkit for web applications.
+
+=item L<ojo>
+
+Fun oneliners using everything above.
+
+=back
+
+For more documentation see L<Mojolicious::Guides> and the tutorial in
+L<Mojolicious::Lite>!
+
 =head1 ATTRIBUTES
 
 L<Mojolicious> inherits all attributes from L<Mojo> and implements the
@@ -371,16 +509,16 @@ following new ones.
 
 =head2 C<controller_class>
 
-    my $class = $mojo->controller_class;
-    $mojo     = $mojo->controller_class('Mojolicious::Controller');
+    my $class = $app->controller_class;
+    $app      = $app->controller_class('Mojolicious::Controller');
 
 Class to be used for the default controller, defaults to
 L<Mojolicious::Controller>.
 
 =head2 C<mode>
 
-    my $mode = $mojo->mode;
-    $mojo    = $mojo->mode('production');
+    my $mode = $app->mode;
+    $app     = $app->mode('production');
 
 The operating mode for your application.
 It defaults to the value of the environment variable C<MOJO_MODE> or
@@ -401,8 +539,8 @@ to your application named C<$mode_mode>.
 
 =head2 C<plugins>
 
-    my $plugins = $mojo->plugins;
-    $mojo       = $mojo->plugins(Mojolicious::Plugins->new);
+    my $plugins = $app->plugins;
+    $app        = $app->plugins(Mojolicious::Plugins->new);
 
 The plugin loader, by default a L<Mojolicious::Plugins> object.
 You can usually leave this alone, see L<Mojolicious::Plugin> if you want to
@@ -410,20 +548,20 @@ write a plugin.
 
 =head2 C<renderer>
 
-    my $renderer = $mojo->renderer;
-    $mojo        = $mojo->renderer(MojoX::Renderer->new);
+    my $renderer = $app->renderer;
+    $app         = $app->renderer(Mojolicious::Renderer->new);
 
-Used in your application to render content, by default a L<MojoX::Renderer>
-object.
+Used in your application to render content, by default a
+L<Mojolicious::Renderer> object.
 The two main renderer plugins L<Mojolicious::Plugin::EpRenderer> and
 L<Mojolicious::Plugin::EplRenderer> contain more specific information.
 
 =head2 C<routes>
 
-    my $routes = $mojo->routes;
-    $mojo      = $mojo->routes(MojoX::Dispatcher::Routes->new);
+    my $routes = $app->routes;
+    $app       = $app->routes(Mojolicious::Routes->new);
 
-The routes dispatcher, by default a L<MojoX::Dispatcher::Routes> object.
+The routes dispatcher, by default a L<Mojolicious::Routes> object.
 You use this in your startup method to define the url endpoints for your
 application.
 
@@ -436,32 +574,40 @@ application.
 
 =head2 C<secret>
 
-    my $secret = $mojo->secret;
-    $mojo      = $mojo->secret('passw0rd');
+    my $secret = $app->secret;
+    $app       = $app->secret('passw0rd');
 
 A secret passphrase used for signed cookies and the like, defaults to the
 application name which is not very secure, so you should change it!!!
 As long as you are using the unsecure default there will be debug messages in
 the log file reminding you to change your passphrase.
 
+=head2 C<session>
+
+    my $session = $app->session;
+    $app        = $app->session(Mojolicious::Session->new);
+
+Simple singed cookie based sessions, by default a L<Mojolicious::Session>
+object.
+
 =head2 C<static>
 
-    my $static = $mojo->static;
-    $mojo      = $mojo->static(MojoX::Dispatcher::Static->new);
+    my $static = $app->static;
+    $app       = $app->static(Mojolicious::Static->new);
 
 For serving static assets from your C<public> directory, by default a
-L<MojoX::Dispatcher::Static> object.
+L<Mojolicious::Static> object.
 
 =head2 C<types>
 
-    my $types = $mojo->types;
-    $mojo     = $mojo->types(MojoX::Types->new);
+    my $types = $app->types;
+    $app      = $app->types(Mojolicious::Types->new);
 
 Responsible for tracking the types of content you want to serve in your
-application, by default a L<MojoX::Types> object.
+application, by default a L<Mojolicious::Types> object.
 You can easily register new types.
 
-    $mojo->types->type(vti => 'help/vampire');
+    $app->types->type(vti => 'help/vampire');
 
 =head1 METHODS
 
@@ -470,46 +616,130 @@ new ones.
 
 =head2 C<new>
 
-    my $mojo = Mojolicious->new;
+    my $app = Mojolicious->new;
 
 Construct a new L<Mojolicious> application.
 Will automatically detect your home directory and set up logging based on
 your current operating mode.
 Also sets up the renderer, static dispatcher and a default set of plugins.
 
+=head2 C<defaults>
+
+    my $defaults = $app->defaults;
+    my $foo      = $app->defaults('foo');
+    $app         = $app->defaults({foo => 'bar'});
+    $app         = $app->defaults(foo => 'bar');
+
+Default values for the stash.
+Note that this method is EXPERIMENTAL and might change without warning!
+
+    $app->defaults->{foo} = 'bar';
+    my $foo = $app->defaults->{foo};
+    delete $app->defaults->{foo};
+
 =head2 C<dispatch>
 
-    $mojo->dispatch($c);
+    $app->dispatch($c);
 
 The heart of every Mojolicious application, calls the static and routes
-dispatchers for every request.
-
-=head2 C<finish>
-
-    $mojo->finish($c);
-
-Clean up after processing a request, usually called automatically.
+dispatchers for every request and passes them a L<Mojolicious::Controller>
+object.
 
 =head2 C<handler>
 
-    $tx = $mojo->handler($tx);
+    $tx = $app->handler($tx);
 
 Sets up the default controller and calls process for every request.
 
+=head2 C<helper>
+
+    $app->helper(foo => sub { ... });
+
+Add a new helper that will be available as a method of the controller object
+and the application object, as well as a function in C<ep> templates.
+Note that this method is EXPERIMENTAL and might change without warning!
+
+    # Helper
+    $app->helper(add => sub { $_[1] + $_[2] });
+
+    # Controller/Application
+    my $result = $self->add(2, 3);
+
+    # Template
+    <%= add 2, 3 %>
+
+=head2 C<hook>
+
+    $app->hook(after_dispatch => sub { ... });
+
+Add hooks to named events.
+Note that this method is EXPERIMENTAL and might change without warning!
+
+The following events are available and run in the listed order.
+
+=over 4
+
+=item after_build_tx
+
+Triggered right after the transaction is built and before the HTTP request
+gets parsed.
+One use case would be upload progress bars.
+(Passed the transaction and application instances)
+
+    $app->hook(before_request => sub {
+        my ($tx, $app) = @_;
+    });
+
+=item before_dispatch
+
+Triggered right before the static and routes dispatchers start their work.
+(Passed the default controller instance)
+
+    $app->hook(before_dispatch => sub {
+        my $self = shift;
+    });
+
+=item after_static_dispatch
+
+Triggered after the static dispatcher determined if a static file should be
+served and before the routes dispatcher starts its work, the callbacks of
+this hook run in reverse order.
+(Passed the default controller instance)
+
+    $app->hook(after_static_dispatch => sub {
+        my $self = shift;
+    });
+
+=item after_dispatch
+
+Triggered after the static and routes dispatchers are finished and a response
+has been rendered, the callbacks of this hook run in reverse order.
+(Passed the current controller instance)
+
+    $app->hook(after_dispatch => sub {
+        my $self = shift;
+    });
+
+=back
+
 =head2 C<plugin>
 
-    $mojo->plugin('something');
-    $mojo->plugin('something', foo => 23);
-    $mojo->plugin('something', {foo => 23});
+    $app->plugin('something');
+    $app->plugin('something', foo => 23);
+    $app->plugin('something', {foo => 23});
+    $app->plugin('Foo::Bar');
+    $app->plugin('Foo::Bar', foo => 23);
+    $app->plugin('Foo::Bar', {foo => 23});
 
 Load a plugin.
+Note that this method is EXPERIMENTAL and might change without warning!
 
 =head2 C<process>
 
-    $mojo->process($c);
+    $app->process($c);
 
 This method can be overloaded to do logic on a per request basis, by default
-just calls dispatch.
+just calls dispatch and passes it a L<Mojolicious::Controller> object.
 Generally you will use a plugin or controller instead of this, consider it
 the sledgehammer in your toolbox.
 
@@ -528,7 +758,7 @@ application.
 
 =head2 C<startup>
 
-    $mojo->startup;
+    $app->startup;
 
 This is your main hook into the application, it will be called at application
 startup.
@@ -557,17 +787,36 @@ startup.
 
     http://github.com/kraih/mojo
 
+=head1 CODE NAMES
+
+Every major release of L<Mojolicious> has a code name, these are the ones
+that have been used in the past.
+
+0.999930, C<Hot Beverage> (u2615)
+
+0.999927, C<Comet> (u2604)
+
+0.999920, C<Snowman> (u2603)
+
 =head1 AUTHOR
 
 Sebastian Riedel, C<sri@cpan.org>.
 
-=head1 CORE DEVELOPERS
+=head1 CORE DEVELOPERS EMERITUS
+
+Retired members of the core team, we thank you dearly for your service.
+
+=over 4
 
 Viacheslav Tykhanovskyi, C<vti@cpan.org>.
 
+=back
+
 =head1 CREDITS
 
-In alphabetical order:
+In alphabetical order.
+
+=over 4
 
 Adam Kennedy
 
@@ -589,6 +838,8 @@ Andy Grundman
 
 Aristotle Pagaltzis
 
+Ashley Dev
+
 Ask Bjoern Hansen
 
 Audrey Tang
@@ -599,14 +850,22 @@ Burak Gursoy
 
 Ch Lamprecht
 
+Chas. J. Owens IV
+
 Christian Hansen
 
 Curt Tilmes
 
+Danijel Tasov
+
 David Davis
 
+Dmitriy Shalashov
+
 Dmitry Konstantinov
 
+Eugene Toropov
+
 Gisle Aas
 
 Glen Hinkle
@@ -617,10 +876,14 @@ Hideki Yamamura
 
 James Duncan
 
+Jan Jona Javorsek
+
 Jaroslav Muhin
 
 Jesse Vincent
 
+John Kingsley
+
 Jonathan Yu
 
 Kazuhiro Shibuya
@@ -645,14 +908,22 @@ Maxim Vuets
 
 Mirko Westermeier
 
+Mons Anderson
+
 Oleg Zhelo
 
 Pascal Gaudette
 
+Paul Tomlin
+
 Pedro Melo
 
+Peter Edwards
+
 Pierre-Yves Ritschard
 
+Quentin Carbonneaux
+
 Rafal Pocztarski
 
 Randal Schwartz
@@ -665,18 +936,32 @@ Sascha Kiefer
 
 Sergey Zasenko
 
+Simon Bertrang
+
 Shu Cho
 
 Stanis Trendelenburg
 
 Tatsuhiko Miyagawa
 
+The Perl Foundation
+
+Tomas Znamenacek
+
+Ulrich Habel
+
+Ulrich Kautz
+
 Uwe Voelker
 
 Yaroslav Korshak
 
 Yuki Kimoto
 
+Zak B. Elep
+
+=back
+
 =head1 COPYRIGHT AND LICENSE
 
 Copyright (C) 2008-2010, Sebastian Riedel.
