@@ -1,5 +1,3 @@
-# Copyright (C) 2008-2010, Sebastian Riedel.
-
 package Mojolicious::Lite;
 
 use strict;
@@ -7,12 +5,11 @@ use warnings;
 
 use base 'Mojolicious';
 
+# Since when is the Internet all about robbing people of their privacy?
+# August 6, 1991.
 use File::Spec;
 use FindBin;
 
-# Make reloading work
-BEGIN { $INC{$0} = $0 }
-
 # It's the future, my parents, my co-workers, my girlfriend,
 # I'll never see any of them ever again... YAHOOO!
 sub import {
@@ -30,91 +27,31 @@ sub import {
 
     # Initialize routes
     my $routes = $app->routes;
-
-    # Route generator
-    my $route = sub {
-        my ($methods, @args) = @_;
-
-        my ($cb, $constraints, $defaults, $name, $pattern);
-        my $conditions = [];
-
-        # Route information
-        my $condition;
-        while (my $arg = shift @args) {
-
-            # Condition can be everything
-            if ($condition) {
-                push @$conditions, $condition => $arg;
-                $condition = undef;
-            }
-
-            # First scalar is the pattern
-            elsif (!ref $arg && !$pattern) { $pattern = $arg }
-
-            # Scalar
-            elsif (!ref $arg && @args) { $condition = $arg }
-
-            # Last scalar is the route name
-            elsif (!ref $arg) { $name = $arg }
-
-            # Callback
-            elsif (ref $arg eq 'CODE') { $cb = $arg }
-
-            # Constraints
-            elsif (ref $arg eq 'ARRAY') { $constraints = $arg }
-
-            # Defaults
-            elsif (ref $arg eq 'HASH') { $defaults = $arg }
-        }
-
-        # Defaults
-        $constraints ||= [];
-
-        # Defaults
-        $defaults ||= {};
-        $defaults = {%$defaults, cb => $cb} if $cb;
-
-        # Name
-        $name ||= '';
-
-        # Create bridge
-        return $routes =
-          $app->routes->bridge($pattern, {@$constraints})->over($conditions)
-          ->to($defaults)->name($name)
-          if !ref $methods && $methods eq 'under';
-
-        # WebSocket
-        my $websocket = 1 if !ref $methods && $methods eq 'websocket';
-        $methods = [] if $websocket;
-
-        # Create route
-        my $route =
-          $routes->route($pattern, {@$constraints})->over($conditions)
-          ->via($methods)->to($defaults)->name($name);
-
-        # WebSocket
-        $route->websocket if $websocket;
-
-        return $route;
-    };
+    $routes->namespace('');
 
     # Prepare exports
     my $caller = caller;
     no strict 'refs';
     no warnings 'redefine';
 
-    # Default template class
+    # Default static and template class
+    $app->static->default_static_class($caller);
     $app->renderer->default_template_class($caller);
 
+    # Root
+    my $root = $routes;
+
     # Export
     *{"${caller}::new"} = *{"${caller}::app"} = sub {$app};
-    *{"${caller}::any"} = sub { $route->(ref $_[0] ? shift : [], @_) };
-    *{"${caller}::get"} = sub { $route->('get', @_) };
+    *{"${caller}::any"} = sub { $routes->any(@_) };
+    *{"${caller}::del"} = sub { $routes->del(@_) };
+    *{"${caller}::get"} = sub { $routes->get(@_) };
     *{"${caller}::under"} = *{"${caller}::ladder"} =
-      sub { $route->('under', @_) };
+      sub { $routes = $root->under(@_) };
     *{"${caller}::plugin"}    = sub { $app->plugin(@_) };
-    *{"${caller}::post"}      = sub { $route->('post', @_) };
-    *{"${caller}::websocket"} = sub { $route->('websocket', @_) };
+    *{"${caller}::post"}      = sub { $routes->post(@_) };
+    *{"${caller}::put"}       = sub { $routes->put(@_) };
+    *{"${caller}::websocket"} = sub { $routes->websocket(@_) };
 
     # We are most likely the app in a lite environment
     $ENV{MOJO_APP} = $app;
@@ -147,7 +84,7 @@ Mojolicious::Lite - Micro Web Framework
 
 =head1 DESCRIPTION
 
-L<Mojolicous::Lite> is a micro web framework built around L<Mojolicious>.
+L<Mojolicious::Lite> is a micro web framework built around L<Mojolicious>.
 
 A minimal Hello World application looks like this, L<strict> and L<warnings>
 are automatically enabled and a few functions imported when you use
@@ -164,7 +101,7 @@ application.
 
 There is also a helper command to generate a small example application.
 
-    % mojolicious generate lite_app
+    % mojo generate lite_app
 
 All the normal L<Mojolicious> command options are available from the command
 line.
@@ -177,9 +114,6 @@ will just work without commands.
     % ./myapp.pl daemon --listen http://*:8080
     Server available at http://127.0.0.1:8080.
 
-    % ./myapp.pl daemon_prefork
-    Server available at http://127.0.0.1:3000.
-
     % ./myapp.pl cgi
     ...CGI output...
 
@@ -202,6 +136,8 @@ option, so you don't have to restart the server after every change.
 
 Routes are basically just fancy paths that can contain different kinds of
 placeholders.
+C<$self> is an instance of L<Mojolicious::Controller> containing both the
+HTTP request and response.
 
     # /foo
     get '/foo' => sub {
@@ -212,13 +148,14 @@ placeholders.
 All routes can have a name associated with them, this allows automatic
 template detection and back referencing with C<url_for>, C<link_to> and
 C<form_for>.
-Names are always the last argument.
+Names are always the last argument, the value C<*> means that the name is
+simply equal to the route without non-word characters.
 
     # /
     get '/' => 'index';
 
     # /foo
-    get '/foo' => 'foo';
+    get '/foo' => '*';
 
     # /bar
     get '/bar' => sub {
@@ -229,8 +166,8 @@ Names are always the last argument.
     __DATA__
 
     @@ index.html.ep
-    <%= link_to foo => {%>Foo<%}%>.
-    <%= link_to bar => {%>Bar<%}%>.
+    <%= link_to Foo => 'foo' %>.
+    <%= link_to Bar => 'bar' %>.
 
     @@ foo.html.ep
     <a href="<%= url_for 'index' %>">Home</a>.
@@ -262,15 +199,15 @@ Template blocks can be reused like functions in Perl scripts.
     __DATA__
 
     @@ block.html.ep
-    <% my $link = {%>
+    <% my $link = begin %>
         <% my ($url, $name) = @_; %>
-        Try <%= link_to $url => {%><%= $name %><%}%>!
-    <%}%>
+        Try <%= link_to $url => begin %><%= $name %><% end %>!
+    <% end %>
     <!doctype html><html>
         <head><title>Sebastians Frameworks!</title></head>
         <body>
-            <%= $link->('http://mojolicious.org', 'Mojolicious') %>
-            <%= $link->('http://catalystframework.org', 'Catalyst') %>
+            <%== $link->('http://mojolicious.org', 'Mojolicious') %>
+            <%== $link->('http://catalystframework.org', 'Catalyst') %>
         </body>
     </html>
 
@@ -287,18 +224,26 @@ other.
 
     @@ first.html.ep
     <!doctype html><html>
-        <head><%= content header => {%><title>Hi!</title><%}%></head>
-        <body><%= content body => {%>First page!<%}%></body>
+        <head>
+            <%= content header => begin %>
+                <title>Hi!</title>
+            <% end %>
+        </head>
+        <body>
+            <%= content body => begin %>
+                First page!
+            <% end %>
+        </body>
     </html>
 
     @@ second.html.ep
     % extends 'first';
-    <% content header => {%>
+    <% content header => begin %>
         <title>Howdy!</title>
-    <%}%>
-    <% content body => {%>
+    <% end %>
+    <% content body => begin %>
         Second page!
-    <%}%>
+    <% end %>
 
 Route placeholders allow capturing parts of a request path until a C</> or
 C<.> separator occurs, results will be stored by name in the C<stash> and
@@ -376,8 +321,15 @@ Routes can be restricted to specific request methods.
 All placeholders get compiled to a regex internally, with regex constraints
 this process can be easily customized.
 
-    # /*
-    any '/:bar' => [bar => qr/\d+/] => sub {
+    # /* (digits)
+    any '/:foo' => [foo => qr/\d+/] => sub {
+        my $self = shift;
+        my $foo  = $self->param('foo');
+        $self->render(text => "Our :foo placeholder matched $foo");
+    };
+
+    # /* (everything else)
+    any '/:bar' => [bar => qr/.*/] => sub {
         my $self = shift;
         my $bar  = $self->param('bar');
         $self->render(text => "Our :bar placeholder matched $bar");
@@ -437,17 +389,19 @@ multiple features at once.
     @@ index.html.ep
     % layout 'funky';
     Who is groovy?
-    <%= form_for test => (method => 'post') => {%>
-        <%= input 'groovy', type => 'text' %>
-        <input type="submit" value="Woosh!" />
-    <%}%>
+    <%= form_for test => (method => 'post') => begin %>
+        <%= text_field 'groovy' %>
+        <%= submit_button 'Woosh!' %>
+    <% end %>
 
     @@ welcome.html.ep
     <%= $groovy %> is groovy!
     <%= include 'menu' %>
 
     @@ menu.html.ep
-    <%= link_to index => {%>Try again<%}%>
+    <%= link_to index => begin %>
+        Try again
+    <% end %>
 
     @@ layouts/funky.html.ep
     <!doctype html><html>
@@ -488,6 +442,21 @@ true value.
     @@ index.html.ep
     Hi Bender!
 
+Prefixing multiple routes is another good use for C<under>.
+
+    use Mojolicious::Lite;
+
+    # /foo
+    under '/foo';
+
+    # GET /foo/bar
+    get '/bar' => sub { shift->render(text => 'bar!') };
+
+    # GET /foo/baz
+    get '/baz' => sub { shift->render(text => 'baz!') };
+
+    app->start;
+
 Conditions such as C<agent> allow even more powerful route constructs.
 
     # /foo
@@ -524,8 +493,9 @@ Formats can be automatically detected by looking at file extensions.
 
 Signed cookie based sessions just work out of the box as soon as you start
 using them.
-The C<flash> can be used to store values that will only be available for one
-request, this is very useful in combination with C<redirect_to>.
+The C<flash> can be used to store values that will only be available for the
+next request (unlike C<stash>, which is only available for the current
+request), this is very useful in combination with C<redirect_to>.
 
     use Mojolicious::Lite;
 
@@ -562,16 +532,16 @@ request, this is very useful in combination with C<redirect_to>.
 
     @@ login.html.ep
     % layout 'default';
-    <%= form_for login => {%>
+    <%= form_for login => begin %>
         <% if (param 'name') { %>
             <b>Wrong name or password, please try again.</b><br />
         <% } %>
         Name:<br />
-        <%= input name => (type => 'text') %><br />
+        <%= text_field 'name' %><br />
         Password:<br />
-        <%= input pass => (type => 'text') %><br />
-        <input type="submit" value="Login" />
-    <%}%>
+        <%= password_field 'pass' %><br />
+        <%= submit_button 'Login' %>
+    <% end %>
 
     @@ index.html.ep
     % layout 'default';
@@ -579,15 +549,17 @@ request, this is very useful in combination with C<redirect_to>.
         <b><%= $message %></b><br />
     <% } %>
     Welcome <%= session 'name' %>!<br />
-    <%= link_to logout => {%>Logout<%}%>
+    <%= link_to logout => begin %>
+        Logout
+    <% end %>
 
 Note that you should use a custom C<secret> to make signed cookies really secure.
 
     app->secret('My secret passphrase here!');
 
 A full featured HTTP 1.1 and WebSocket client is built right in.
-Especially in combination with L<Mojo::JSON> this can be a very powerful
-tool.
+Especially in combination with L<Mojo::JSON> and L<Mojo::DOM> this can be a
+very powerful tool.
 
     get '/test' => sub {
         my $self = shift;
@@ -599,7 +571,7 @@ WebSocket applications have never been this easy before.
 
     websocket '/echo' => sub {
         my $self = shift;
-        $self->receive_message(sub {
+        $self->on_message(sub {
             my ($self, $message) = @_;
             $self->send_message("echo: $message");
         });
@@ -616,14 +588,22 @@ directory.
         $self->render('foo/bar');
     };
 
-Static files will be automatically served from the C<public> directory if it
-exists.
+Static files will be automatically served from the C<DATA> section
+(even Base 64 encoded) or a C<public> directory if it exists.
+
+    @@ something.js
+    alert('hello!');
+
+    @@ test.txt (base64)
+    dGVzdCAxMjMKbGFsYWxh
 
     % mkdir public
     % mv something.js public/something.js
 
 Testing your application is as easy as creating a C<t> directory and filling
-it with normal Perl unit tests like C<t/funky.t>.
+it with normal Perl unit tests.
+Some plugins depend on the actual script name, so a test file for the
+application C<myapp.pl> should be named C<t/myapp.t>.
 
     use Test::More tests => 3;
     use Test::Mojo;
@@ -639,15 +619,15 @@ Run all unit tests with the C<test> command.
 
     % ./myapp.pl test
 
-To make your tests less noisy you can also change the application log level
-directly in your test files.
+To make your tests more noisy and show you all log messages you can also
+change the application log level directly in your test files.
 
-    app->log->level('error');
+    $t->app->log->level('debug');
 
 To disable debug messages later in a production setup you can change the
 L<Mojolicious> mode, default will be C<development>.
 
-    % MOJO_MODE=production ./myapp.pl
+    % ./myapp.pl --mode production
 
 Log messages will be automatically written to a C<log/$mode.log> file if a
 C<log> directory exists.
@@ -686,10 +666,82 @@ L<Mojolicious::Lite> and L<Mojolicious> applications.
 Both share about 99% of the same code, so almost everything you learned in
 this tutorial applies there too. :)
 
-    % mojolicious generate app
+    % mojo generate app
 
 Have fun!
 
+=head1 FUNCTIONS
+
+L<Mojolicious::Lite> implements the following functions.
+
+=head2 C<any>
+
+    my $route = any '/:foo' => sub {...};
+    my $route = any [qw/get post/] => '/:foo' => sub {...};
+
+Generate route matching any of the listed HTTP request methods or all.
+See also the tutorial above for more argument variations.
+
+=head2 C<app>
+
+    my $app = app;
+
+The L<Mojolicious::Lite> application.
+
+=head2 C<del>
+
+    my $route = del '/:foo' => sub {...};
+
+Generate route matching only C<DELETE> requests.
+See also the tutorial above for more argument variations.
+
+=head2 C<get>
+
+    my $route = get '/:foo' => sub {...};
+
+Generate route matching only C<GET> requests.
+See also the tutorial above for more argument variations.
+
+=head2 C<plugin>
+
+    plugin 'something';
+    plugin 'something', foo => 23;
+    plugin 'something', {foo => 23};
+    plugin 'Foo::Bar';
+    plugin 'Foo::Bar', foo => 23;
+    plugin 'Foo::Bar', {foo => 23};
+
+Load a plugin.
+
+=head2 C<post>
+
+    my $route = post '/:foo' => sub {...};
+
+Generate route matching only C<POST> requests.
+See also the tutorial above for more argument variations.
+
+=head2 C<put>
+
+    my $route = put '/:foo' => sub {...};
+
+Generate route matching only C<PUT> requests.
+See also the tutorial above for more argument variations.
+
+=head2 C<under>
+
+    my $route = under sub {...};
+    my $route = under '/:foo';
+
+Generate bridge to which all following routes are automatically appended.
+See also the tutorial above for more argument variations.
+
+=head2 C<websocket>
+
+    my $route = websocket '/:foo' => sub {...};
+
+Generate route matching only C<WebSocket> handshakes.
+See also the tutorial above for more argument variations.
+
 =head1 ATTRIBUTES
 
 L<Mojolicious::Lite> inherits all attributes from L<Mojolicious>.
